@@ -6,7 +6,6 @@
 #                                                                           #
 # The full license is in the file LICENSE, distributed with this software.  #
 #############################################################################
-
 import gettext
 import io
 import sys
@@ -45,12 +44,12 @@ from jupyter_server.config_manager import recursive_update
 from jupyter_server.utils import url_path_join, run_sync
 from jupyter_server.services.config import ConfigManager
 
+from jupyterlab_server.themes_handler import ThemesHandler
+
 from jupyter_client.kernelspec import KernelSpecManager
 
 from jupyter_core.paths import jupyter_config_path, jupyter_path
 
-from ipython_genutils.py3compat import getcwd
-
 from .paths import ROOT, STATIC_ROOT, collect_template_paths, collect_static_paths
 from .handler import VoilaHandler
 from .treehandler import VoilaTreeHandler
@@ -60,6 +59,9 @@ from .configuration import VoilaConfiguration
 from .execute import VoilaExecutor
 from .exporter import VoilaExporter
 from .shutdown_kernel_handler import VoilaShutdownKernelHandler
+from .voila_kernel_manager import voila_kernel_manager_factory
+from .request_info_handler import RequestInfoSocketHandler
+from .utils import create_include_assets_functions
 
 _kernel_id_regex = r"(?P<kernel_id>\w+-\w+-\w+-\w+-\w+)"
 
@@ -130,6 +132,8 @@ class Voila(Application):
         'server_url': 'Voila.server_url',
         'enable_nbextensions': 'VoilaConfiguration.enable_nbextensions',
         'show_tracebacks': 'VoilaConfiguration.show_tracebacks',
+        'preheat_kernel': 'VoilaConfiguration.preheat_kernel',
+        'pool_size': 'VoilaConfiguration.default_pool_size'
     }
     classes = [
         VoilaConfiguration,
@@ -316,7 +320,7 @@ class Voila(Application):
         if self.notebook_path:
             return os.path.dirname(os.path.abspath(self.notebook_path))
         else:
-            return getcwd()
+            return os.getcwd()
 
     def _init_asyncio_patch(self):
         """set default asyncio policy to be compatible with tornado
@@ -409,7 +413,19 @@ class Voila(Application):
             parent=self
         )
 
-        self.kernel_manager = self.voila_configuration.multi_kernel_manager_class(
+        # we create a config manager that load both the serverconfig and nbconfig (classical notebook)
+        read_config_path = [os.path.join(p, 'serverconfig') for p in jupyter_config_path()]
+        read_config_path += [os.path.join(p, 'nbconfig') for p in jupyter_config_path()]
+        self.config_manager = ConfigManager(parent=self, read_config_path=read_config_path)
+        self.contents_manager = LargeFileManager(parent=self)
+        preheat_kernel: bool = self.voila_configuration.preheat_kernel
+        pool_size: int = self.voila_configuration.default_pool_size
+        kernel_manager_class = voila_kernel_manager_factory(
+            self.voila_configuration.multi_kernel_manager_class,
+            preheat_kernel,
+            pool_size
+        )
+        self.kernel_manager = kernel_manager_class(
             parent=self,
             connection_dir=self.connection_dir,
             kernel_spec_manager=self.kernel_spec_manager,
@@ -427,12 +443,6 @@ class Voila(Application):
         env = jinja2.Environment(loader=jinja2.FileSystemLoader(self.template_paths), extensions=['jinja2.ext.i18n'], **jenv_opt)
         nbui = gettext.translation('nbui', localedir=os.path.join(ROOT, 'i18n'), fallback=True)
         env.install_gettext_translations(nbui, newstyle=False)
-        self.contents_manager = LargeFileManager(parent=self)
-
-        # we create a config manager that load both the serverconfig and nbconfig (classical notebook)
-        read_config_path = [os.path.join(p, 'serverconfig') for p in jupyter_config_path()]
-        read_config_path += [os.path.join(p, 'nbconfig') for p in jupyter_config_path()]
-        self.config_manager = ConfigManager(parent=self, read_config_path=read_config_path)
 
         # default server_url to base_url
         self.server_url = self.server_url or self.base_url
@@ -471,9 +481,26 @@ class Voila(Application):
                     'default_filename': 'index.html'
                 },
             ),
+            (
+                url_path_join(self.server_url, r'/voila/themes/(.*)'),
+                ThemesHandler,
+                {
+                    'themes_url': '/voila/themes',
+                    'path': '',
+                    'labextensions_path': jupyter_path('labextensions'),
+                    'no_cache_paths': ['/']
+                },
+            ),
             (url_path_join(self.server_url, r'/voila/api/shutdown/(.*)'), VoilaShutdownKernelHandler)
         ])
 
+        if preheat_kernel:
+            handlers.append(
+                (
+                    url_path_join(self.server_url, r'/voila/query/%s' % _kernel_id_regex),
+                    RequestInfoSocketHandler
+                )
+            )
         # Serving notebook extensions
         if self.voila_configuration.enable_nbextensions:
             handlers.append(
@@ -524,7 +551,7 @@ class Voila(Application):
                      'template_paths': self.template_paths,
                      'config': self.config,
                      'voila_configuration': self.voila_configuration
-                 }),
+                }),
             ])
 
         self.app.add_handlers('.*$', handlers)
@@ -549,7 +576,7 @@ class Voila(Application):
         success = False
         for port in self.random_ports(self.port, self.port_retries+1):
             try:
-                self.app.listen(port)
+                self.app.listen(port, self.ip)
                 self.port = port
                 self.log.info('Voilà is running at:\n%s' % self.display_url)
             except socket.error as e:
@@ -601,9 +628,15 @@ class Voila(Application):
             #     url = url_concat(url, {'token': self.token})
             url = url_path_join(self.connection_url, uri)
 
+            include_assets_functions = create_include_assets_functions(self.voila_configuration.template, url)
+
             jinja2_env = self.app.settings['jinja2_env']
             template = jinja2_env.get_template('browser-open.html')
-            fh.write(template.render(open_url=url, base_url=url))
+            fh.write(template.render(
+                open_url=url, base_url=url,
+                theme=self.voila_configuration.theme,
+                **include_assets_functions
+            ))
 
         def target():
             return browser.open(urljoin('file:', pathname2url(open_file)), new=self.webbrowser_open_new)
